import IKDBQuery from './IKDBQuery';
import { convertToAttr, marshall, unmarshall } from "@aws-sdk/util-dynamodb";
//
export default class IKDBQueryPut extends IKDBQuery {
  constructor() {
    super();
    this.putItems = {};
  }
  /*public methods*/
  //return if the value was appended or not
  appendValue(source, key, isObject) {
    //check for optional value
    if (isObject == undefined) isObject = false;
    //check if value is valid
    if (source[key] != undefined &&
       ((Object.keys(source[key]).length > 0) ||
        (!isNaN(source[key]) && source[key].length != 0) ||
        (source[key].length > 0))) {
      //Check for object conversion
      let val;
      if (isObject) val = {"M": marshall(source[key], {removeUndefinedValues: true, convertClassInstanceToMap: true })};
      else val = convertToAttr(source[key], {removeUndefinedValues: true, convertClassInstanceToMap: true });
      //valid
      if (val != undefined) { this.putItems[key]= val; }
      // console.debug("Apppending key:", key, " val:", val);
      return true;
    } return false;
  }
  setItems(items) { this.putItems = items; }
  async run(dbManager) {
    const localConsole = (dbManager.transaction ? dbManager.transaction.logger : console);
    const query = this.__rawQuery(dbManager.tableName);
    localConsole.log('Putting item: ', query);
    const resp = await dbManager.connection.putItem(query);
    // localConsole.log('Raw result: ', resp);
    const unmarshalled = resp.Items.map(unmarshall);
    localConsole.log('Parsed result: ', unmarshalled);
    return unmarshalled;
  }

  /*private methods*/
  __rawQuery(dbTableName) {
    return { ...this._rawQuery(dbTableName),
             Item: this.putItems, ReturnValues: "ALL_NEW" /*request new values added*/ };
  }
}
