import IKGlobals from './../IKGlobals';
import Utils from './../API/IKUtils';
import IKResponse, { IKBadRequestResponse } from './../API/IKResponse';
//https://www.npmjs.com/package/superstruct
import { struct, superstruct } from 'superstruct';
//validation libs
import isUuid from 'is-uuid';
import isEmail from 'is-email';
//
export default class IKValidation {
  constructor(config) {
    this._buildInternalTypes((config && config.additionalTypes ? config.additionalTypes : {}));
  }
  _buildInternalTypes(additionalTypes) {
    this.structor = superstruct({
      types: {
        ...IKValidation.internalTypes(),
        ...additionalTypes
      }
    });
  }
  async validate(spec, obj, request) {
    let parsedObj = null;
    let err = null;
    try {
      let parsedRawObj = obj;
      parsedObj = this.structor(spec)(parsedRawObj);
    } catch (e) {
      console.debug(e);
      const { path, value, type } = e;
      const key = (path ? path[0] : undefined);
      let errorMessage = e;
      //Check for value required error
      if (value === undefined) errorMessage = `Required value '${key}' is missing.`
      else if (type === undefined) errorMessage = `Unknown key '${key}'.`
      else errorMessage = `Invalid value '${value}' for key '${key}'.`
      //Generate error
      err = IKBadRequestResponse(IKGlobals.ErrorResponseValidationFail + errorMessage, IKGlobals.ErrorCode_InvalidInput);
    }
    //Catch error as response
    if (!parsedObj && err) return err;
    //run normal code out of this try/catch scope
    return await request(parsedObj);
  }
}
//
IKValidation.internalTypes = function() {
  return {
    optionalNumber: value => (value == undefined || !isNaN(parseInt(value))),
    optionalValidString: value => { return (!value || Utils.isValidString(value) == true); },
    validString: value => { return (Utils.isValidString(value) == true); },
    email: value => { return (isEmail(value) && value.length < 256); },
    fullAddress: value => {
      if (!value) return false;
      return true;
    },
    'date-yyyymmdd': value => {
      if (!value) return false;
      //TODO CHECK DATE FORMAT
      let d = new Date(value);
      return (d instanceof Date && !isNaN(d));
    },
    'arrayIDs': value => {
      if (!value || !Array.isArray(value)) return false;
      for (let id of value) if (id.length < 10) return false;
      return true;
    },
    //TODO: this should be moved out to the project requiring this
    ACEArray: value => {
      if (!value || !value.length) return false;
      for (let val of value) {
        if (!val.accessType || !val.roleID) return false;
      }
      return true;
    },
  }
}